/*---------------------------------------------------------------------------+
| This file is part of Mumail, Copyright (c) 1992-1993 by
| Muhammad M. Saggaf. All rights reserved.
|
| See the file COPYING (1-COPYING) or the manual page mumail(1)
| for a full statement of rights and permissions.
+---------------------------------------------------------------------------*/

/*                               -*- Mode: C -*- 
 * MmMain.c --- Main module
 * Author          : Muhammad M. Saggaf
 * Created On      : April 1993
 * Last Modified By: system admin
 * Last Modified On: Thu Jul  1 00:32:11 1993
 * Update Count    : 47
 * Status          : Mostly OK, needs some cleaning up
 */

#include "config.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/AsciiText.h>
#if USE_3D_FRAME
#include "Xfwf/Frame.h"
#endif

#include "MmDecl.h"
#include "MuWin.h"
#include "MuGeneric.h"
#define IS_MAIN
#include "version.h"

extern int
                CompareLettersByDateProc(),
                CompareLettersBySubjectProc(),
                CompareLettersByNameProc();

extern void
                HighlightToEndOFLineAction(),

                BodyBoxNextPageAction(),
                BodyBoxPrevPageAction(),
                BodyBoxNextLineAction(),
                BodyBoxEndOfPageAction(),
               
                CloseWindowAction(),
                IconifyWindowAction(),

                OpenFolderAction(),
                SaveFolderAction(),
                CommitChangesAction(),
                FolderInfoAction(),
                QuickExitAction(),
                ExitAction(),

                DisplayLetterGeneralAction(),
                RedisplayCurLetterPartialAction(),
                RedisplayCurLetterWholeAction(),

                TagUntagLetterAction(),
                RetagLettersAction(),
                DeleteLetterAction(),
                UndeleteLetterAction(),
                ComposeLetterAction(),
                ReplyToLetterAction(),
                ForwardLetterAction(),
                ExternalComposeLetterAction(),
                CopyLetterAction(),
                RefileLetterAction(),
                LetterInfoAction(),

                SendDraftAction(),
                CancelDraftAction(),
                QuoteScStyleFmtAction(),
                QuoteScStyleUnfmtAction(),
                QuoteRnStyleAction(),
                InsertSignatureAction(),

                HighlightCurrentLineAction(),
                PrevLineAction(),
                NextLineAction(),
                BeginningOfBufferAction(),
                EndOfBufferAction(),

                MenuButtonPopdownMenuAction(),
                PopupMenuIfMenuButtonAction();

void
                GetResources();

QueryResources res;

void
main(argc, argv)
     int             argc;
     char           *argv[];
{
  XtAppContext          appContext;  
  Widget                topLevel;

  static char    *fallbackResources[] = {
#include "Mumail.ad.h"
    NULL,
  };

  static XrmOptionDescRec optionList[] =  {
    {"-modems", "modems", XrmoptionSepArg, NULL},
    {"-script", "script", XrmoptionSepArg, NULL},
    {"-entries", "defaultPhoneEntries", XrmoptionSepArg, NULL},
    {"-dial", "dialAutoStart", XrmoptionNoArg, "True"},
    {"-nodial", "dialAutoStart", XrmoptionNoArg, "False"},
    {"-emulator", "emulator", XrmoptionSepArg, ""},
    {"-noemulator", "noemulator", XrmoptionNoArg, ""},
    {"-nodefargs", "nodefargs", XrmoptionNoArg, ""},
  };

  static XtActionsRec actionTable[] = {
    {"ClearCutBuffer", ClearCutBufferAction},
    {"IncCurLetterNumber", IncCurLetterNumberAction},
    {"DecrCurLetterNumber", DecrCurLetterNumberAction},

    {"BodyBoxNextPage", BodyBoxNextPageAction},
    {"BodyBoxPrevPage", BodyBoxPrevPageAction},
    {"BodyBoxNextLine", BodyBoxNextLineAction},
    {"BodyBoxEndOfPage", BodyBoxEndOfPageAction},

    {"CloseWindow", CloseWindowAction},
    {"IconifyWindow", IconifyWindowAction},
    {"HighlightCurrentLine", HighlightCurrentLineAction},

    {"OpenFolder", OpenFolderAction},
    {"SaveFolder", SaveFolderAction},
    {"CommitChanges", CommitChangesAction},
    {"FolderInfo", FolderInfoAction},
    {"QuickExit", QuickExitAction},
    {"Exit", ExitAction},

    {"SetCurLetterNumber", SetCurLetterNumberAction},
    {"DisplayLetterGeneral", DisplayLetterGeneralAction},
    {"RedisplayCurLetterPartial", RedisplayCurLetterPartialAction},
    {"RedisplayCurLetterWhole", RedisplayCurLetterWholeAction},

    {"TagUntagLetter", TagUntagLetterAction},
    {"RetagLetters", RetagLettersAction},
    {"DeleteLetter", DeleteLetterAction},
    {"UndeleteLetter", UndeleteLetterAction},
    {"ComposeLetter", ComposeLetterAction},
    {"ReplyToLetter", ReplyToLetterAction},
    {"ForwardLetter", ForwardLetterAction},
    {"ExternalComposeLetter", ExternalComposeLetterAction},
    {"CopyLetter", CopyLetterAction},
    {"RefileLetter", RefileLetterAction},
    {"LetterInfo", LetterInfoAction},

    {"SendDraft", SendDraftAction},
    {"CancelDraft", CancelDraftAction},
    {"QuoteScStyleFmt", QuoteScStyleFmtAction},
    {"QuoteScStyleUnfmt", QuoteScStyleUnfmtAction},
    {"QuoteRnStyle", QuoteRnStyleAction},
    {"InsertSignature", InsertSignatureAction},

    {"HighlightToEndOFLine", HighlightToEndOFLineAction},
    {"PrevLine", PrevLineAction},
    {"NextLine", NextLineAction},
    {"BeginningOfBuffer", BeginningOfBufferAction},
    {"EndOfBuffer", EndOfBufferAction},


    {"MenuButtonPopdownMenu", MenuButtonPopdownMenuAction},
    {"PopupMenuIfMenuButton", PopupMenuIfMenuButtonAction},
  };

  topLevel = XtAppInitialize(&appContext, "Mumail", optionList,
							 XtNumber(optionList), &argc, argv,
							 fallbackResources, NULL, 0);
  if (argc > 1)	{
	MuError(FmtString1("Incorrect or incomplete argument: %s", argv[1])); 
	 exit(1);
  }
  MuNotice(FmtString2("Mumail %s.%s is initializing..", VERSION, REVISION));  

  XtAppAddActions(appContext, actionTable, XtNumber(actionTable));
  XtOverrideTranslations(topLevel,
		       XtParseTranslationTable("<Message>WM_PROTOCOLS: QuickExit()"));

  GetResources(topLevel);

  AddScreen(topLevel, FileInMailDir(INFOLDER_NAME));

  XtAppMainLoop(appContext);
}

Widget
SetupTocW(parentW)
	 Widget parentW;
{
  Widget          frameW, widget;
  static          XawTextSelectType selectTypes[] = {XawselectLine,
													   XawselectNull};
  
#if USE_3D_FRAME
  frameW = 	XtVaCreateManagedWidget("tocFrame", xfwfFrameWidgetClass,
									parentW, NULL); 
#else
  frameW = parentW;
#endif

  widget = AddAsciiText("toc", frameW);
  XtVaSetValues(widget, XtNselectTypes, selectTypes, NULL);
  return widget;
}

Widget
SetupBodyW(parentW)
	 Widget parentW;
{
  Widget          widget, frameW;
  Widget          textSinkW;
  XFontStruct    *font;
  Dimension       width;

#if USE_3D_FRAME
  frameW = 	XtVaCreateManagedWidget("bodyFrame", xfwfFrameWidgetClass,
									parentW, NULL); 
#else
  frameW = parentW;
#endif

  widget = AddAsciiText("body", frameW);

  XtVaGetValues(widget, XtNfont, &font, XtNtextSink, 
				&textSinkW,	NULL);
  width = font->max_bounds.rbearing + font->min_bounds.lbearing;
  XtVaSetValues(widget, XtNwidth, 81 * width, NULL);
  XtVaSetValues(widget, XtNrightMargin, 10 * width, NULL); 
  return widget;

/*  XtVaSetValues(widget, XtNwidth, 81 * width, XtNleftMargin, 
				width, XtNrightMargin, 10 * width, NULL); 

  XtVaSetValues(textSinkW, XtNwidth, (Dimension)486, 
				XtNleftMargin, 
				(Dimension) 6 , XtNrightMargin,  (Dimension) 60,
				NULL); 
  printf("-%d-\n", (int)width);*/
}

void
SetupDialogW(scn, parentW)
	 SCREEN          *scn;
	 Widget          parentW;
{
  Widget          dialogW, frameW;

#if USE_3D_FRAME
  frameW = 	XtVaCreateManagedWidget("dialogFrame", xfwfFrameWidgetClass,
									parentW, NULL); 
#else
  frameW = parentW;
#endif

  dialogW = AddPaned("dialogW", frameW);

  scn->dialog.yesW = AddButton("yesW", dialogW, NULL, NULL);
  scn->dialog.cancelW = AddButton("cancelW", dialogW, NULL, NULL);
  scn->dialog.valueW = AddAsciiText("valueW", dialogW);

  XtVaSetValues(scn->dialog.yesW, XtNmappedWhenManaged, False, NULL);
  XtVaSetValues(scn->dialog.cancelW, XtNmappedWhenManaged, False, NULL);

  scn->dialog.mode = DIALOG_INACTIVE;
}

void
SetIcon(parentW)
     Widget          parentW;
{
#include "Mumail.icon.h"

  Pixmap icon;

  icon = 
	XCreateBitmapFromData(XtDisplay(parentW),
						  DefaultRootWindow(XtDisplay(parentW)),
						  icon_bits, icon_width, icon_height);
  XtVaSetValues(GetShell(parentW), XtNiconPixmap, icon, NULL);
}

void
DisplayLogo(scn)
	 SCREEN   *scn;
{
#include "Mumail.logo.h"

  char        versionInfo[BUF256];
  String      logoBuffer; 

  sprintf(versionInfo, "%5sVersion %s rev. %s %s-%s %s@%s %s %s\n\n\n%12s%s", 
		  "", VERSION, REVISION, COMPILE_HOSTTYPE, COMPILE_OSNAME, 
		  COMPILE_BY, COMPILE_HOST, COMPILE_DATE, COMPILE_TIME, "",
		  "Comments, suggestions, and bug fixes are welcome");
  
  logoBuffer = XtMalloc(strlen(logo)+strlen(versionInfo));
  sprintf(logoBuffer, "%s%s", logo, versionInfo);

  DisplayBufferInBodyBox(scn, logoBuffer);
  XtFree(logoBuffer);
}

void
DisplayLogoCallback(widget, clientData, callData)
	 Widget    widget;
	 XtPointer clientData;
	 XtPointer callData;
{
  DisplayLogo((SCREEN*)clientData);
}

String
XMailerMessage()
{
  /* Instead of allocating new space */
  String buffer = FmtString0(""); 

  sprintf(buffer, "X-Mailer: Mumail [version %s.%s %s-%s]", 
		  VERSION, REVISION, COMPILE_HOSTTYPE, COMPILE_OSNAME);
  return buffer;
}


/*---------------------------------------------------------------------------+
| GetResources - gets the application resources.
+---------------------------------------------------------------------------*/


int
FigureEnumeration(var, val, num, enumS, enumV)
	 XtPointer          *var;
	 String             val;
     int                num;
	 String             *enumS;
	 XtPointer          *enumV;
{
  int          n;

  for (n = 0; n < num; n++)
	if (CaseNStrSame(val, enumS[n], strlen(enumS[n])))
	  {*var = enumV[n]; return 0;}
  return -1;
}

#define MAX_RES_ENUM 10

void
GetResources(topLevel)
	 Widget          topLevel;
{
  String             enumS[MAX_RES_ENUM];
  XtPointer          enumV[MAX_RES_ENUM], retVal;
  int                n;

#define offset(field) XtOffsetOf(QueryResources, field)
  static XtResource resources[] = {
    {"printCommand", "printCommand", XtRString, sizeof(String),
	   offset(printCommand), XtRString, (XtPointer)"lpr"},
    {"externalEditor", "ExternalEditor", XtRString, sizeof(String),
	   offset(externalEditor), XtRString, (XtPointer)"emacs"},
	{"signatureFile", "SignatureFile", XtRString, sizeof(String),
	   offset(signatureFile), XtRString, (XtPointer)".signature"},

    {"contentInToc", "ContentInToc", XtRBoolean, sizeof(Boolean), 
	   offset(contentInToc), XtRImmediate, (XtPointer)False},
    {"folderBackup", "FolderBackup", XtRBoolean, sizeof(Boolean), 
	   offset(folderBackup), XtRImmediate, (XtPointer)True},
  },

  citeResources[] = {
    {"fillColumn", "FillColumn", XtRInt, sizeof(int),
	   offset(quoteFillColumn), XtRImmediate, (XtPointer)70},
    {"indentString", "IndentString", XtRString, sizeof(String),
	   offset(quoteIndentString), XtRString, (XtPointer)"\t"},
    {"nestedAttribFmt", "NestedAttribFmt", XtRString, sizeof(String),
	   offset(quoteNestedAttribFmt), XtRString, 
	   (XtPointer)"\n>>>>> %s wrote:\n>\n"},
    {"nonNestedAttribFmt", "NonNestedAttribFmt", XtRString, 
	   sizeof(String), offset(quoteNonNestedAttribFmt), XtRString, 
	   (XtPointer)"\n>>>>> %s == %s wrote:\n\n"},
  },

  mailAgentResources[] = {
    {"passAddress", "PassAddress", XtRBoolean, sizeof(Boolean), 
	   offset(mailAgentPassAddress), XtRImmediate, (XtPointer)False},
    {"program", "Program", XtRString, sizeof(String),
	   offset(mailAgentProgram), XtRString, (XtPointer)"/usr/lib/sendmail"},
    {"arguments", "Arguments", XtRString, sizeof(String),
	   offset(mailAgentArgs), XtRString, (XtPointer)"-bm -oi -oem -t"},
  },

  mimeResources[] = {
    {"agentCommand", "AgentCommand", XtRString, sizeof(String), 
	   offset(mimeCommand), XtRString, 
	   (XtPointer)"metamail -d -r -x -m Mumail"},
    {"invokeAgent", "InvokeAgent", XtRString, sizeof(String),
	   offset(invokeMimeS), XtRString, (XtPointer)"ask"},
    {"addHeaders", "AddHeaders", XtRBoolean, sizeof(Boolean),
	   offset(addMimeHeaders), XtRImmediate, (XtPointer)True},
    {"defaultContentType", "DefaultContentType", XtRString, sizeof(String),
	   offset(defaultContentType), XtRString, (XtPointer)"text/plain"},
    {"defaultCharSet", "DefaultCharSet", XtRString, sizeof(String),
	   offset(defaultCharSet), XtRString, (XtPointer)"US-ASCII"},
    {"defaultTransferEncoding", "DefaultTransferEncoding", XtRString, 
	   sizeof(String), offset(defaultTransferEncoding), XtRString, 
	   (XtPointer)"7BIT"},
    {"multipartBoundary", "MultipartBoundary", XtRString, sizeof(String),
	   offset(mimeBoundary), XtRString, (XtPointer)"####"},
  },

  sortResources[] = {
    {"when", "When", XtRString, sizeof(String), 
	   offset(sortWhenS), XtRString, (XtPointer)"burst open"},
    {"majorKey", "MajorKey", XtRString, sizeof(String),
	   offset(majorSortKeyS), XtRString, (XtPointer)"date"},
    {"minorKey", "MinorKey", XtRString, sizeof(String),
	   offset(minorSortKeyS), XtRString, (XtPointer)"subject"},
    {"descending", "descending", XtRBoolean, sizeof(Boolean), 
	   offset(sortDescending), XtRImmediate, (XtPointer)False},
  };
#undef offset

  XtGetApplicationResources(topLevel, (XtPointer)&res, resources,
							XtNumber(resources), NULL, 0);

  XtGetSubresources(topLevel, (XtPointer)&res, "cite", "Cite", 
					citeResources, XtNumber(citeResources), NULL, 0);
  XtGetSubresources(topLevel, (XtPointer)&res, "mailAgent", "MailAgent", 
					mailAgentResources, XtNumber(mailAgentResources), NULL, 0);
  XtGetSubresources(topLevel, (XtPointer)&res, "mime", "Mime", 
					mimeResources, XtNumber(mimeResources), NULL, 0);
  XtGetSubresources(topLevel, (XtPointer)&res, "sort", "Sort", 
					sortResources, XtNumber(sortResources), NULL, 0);

  n = 0;
  enumS[n] = "never"; enumV[n] = (XtPointer)MIME_INVOKE_NEVER; n++;
  enumS[n] = "always"; enumV[n] = (XtPointer)MIME_INVOKE_ALWAYS; n++;
  enumS[n] = "ask"; enumV[n] = (XtPointer)MIME_INVOKE_ASK; n++;
  if (FigureEnumeration(&retVal, (XtPointer)StripStringB(res.invokeMimeS), 
						n, enumS, enumV) < 0)
	res.invokeMime = MIME_INVOKE_ASK;
  else res.invokeMime = (int)(long)retVal;


  res.sortWhen = 0;
  if (CaseStrstr(res.sortWhenS, "open")) res.sortWhen |= SORT_ON_OPEN;
  if (CaseStrstr(res.sortWhenS, "save")) res.sortWhen |= SORT_ON_SAVE;
  if (CaseStrstr(res.sortWhenS, "burst")) res.sortWhen |= SORT_ON_BURST;
  if (CaseStrstr(res.sortWhenS, "inc")) res.sortWhen |= SORT_ON_INC;
  if (CaseStrstr(res.sortWhenS, "commit")) res.sortWhen |= SORT_ON_COMMIT;

  n = 0;
  enumS[n] = "date"; enumV[n] = (XtPointer)CompareLettersByDateProc; n++;
  enumS[n] = "subject"; enumV[n] = (XtPointer)CompareLettersBySubjectProc; 
  n++;
  enumS[n] = "name"; enumV[n] = (XtPointer)CompareLettersByNameProc; n++;
  if (FigureEnumeration(&retVal, (XtPointer)StripStringB(res.majorSortKeyS), 
						n, enumS, enumV) < 0)
	res.majorSortProc = CompareLettersByDateProc;
  else res.majorSortProc = (int(*)())retVal;

  if (FigureEnumeration(&retVal, (XtPointer)StripStringB(res.minorSortKeyS), 
						n, enumS, enumV) < 0)
	res.majorSortProc = CompareLettersBySubjectProc;
  else res.minorSortProc = (int(*)())retVal;
}
